/* File:        $Id: NumberUtils.java 11 2007-07-24 10:11:24Z kfc $
 * Revision:    $Revision: 11 $
 * Author:      $Author: kfc $
 * Date:        $Date: 2007-07-24 12:11:24 +0200 (Tue, 24 Jul 2007) $
 *
 * The Netarchive Suite - Software to harvest and preserve websites
 * Copyright 2004-2007 Det Kongelige Bibliotek and Statsbiblioteket, Denmark
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

package dk.netarkivet.harvester.datamodel;

/**
 * Number related utilities
 *
 */

public class NumberUtils {
    /**
     * Return the smallest value of two given positive longs, with the addition
     * that -1 means infinity
     * @param l1 The first value
     * @param l2 The second value
     * @return Smallest value
     */
    public static long minInf(long l1, long l2) {
        if (l1 != Constants.HERITRIX_MAXBYTES_INFINITY &&
            l2 != Constants.HERITRIX_MAXBYTES_INFINITY) {
            return Math.min(l1, l2);
        } else if (l2 != Constants.HERITRIX_MAXBYTES_INFINITY) {
            return l2;
        } else {
            return l1;
        }
    }

    /**
     * compare two given positive longs, with the addition that
     * -1 means infinity
     * @param l1 The first value
     * @param l2 The second value
     * @return -1 if first value is smallest, 0 if equal, 1 if second value is
     * smallest
     */
    public static int compareInf(long l1, long l2) {
        if (l1 == l2) {
            return 0;
        }
        return minInf(l1,l2) == l1 ? -1 : 1;
    }
}
